import 'dart:math';

import 'package:flutter/material.dart';

/// create by 张风捷特烈 on 2020/7/16
/// contact me by email 1981462002@qq.com
/// 说明: 展开/折叠组件

class Folder extends StatefulWidget {
  final Widget child; //子组件 @required
  final List<Widget> children; //折叠内容
  final Widget tile; //尾部折叠指示组件 - 默认[]
  final int duration; //折叠时长 - 默认200 ms
  final double tileOffsetRight; //指示组件右边距 - 默认10
  final Curve foldCurve; //动画曲线 - 默认Curves.easeInToLinear
  final ValueChanged<bool> onExpansionChanged; //折叠回调
  final bool initiallyExpanded; // 是否一开始展开

  Folder(
      {Key key,
        @required this.child,
        this.initiallyExpanded = false,
        this.tile,
        this.foldCurve = Curves.easeInToLinear,
        this.tileOffsetRight = 10.0,
        this.duration = 200,
        this.onExpansionChanged,
        this.children = const <Widget>[]})
      : super(key: key);

  @override
  _FolderState createState() => _FolderState();
}

class _FolderState extends State<Folder> with SingleTickerProviderStateMixin {
  var _crossFadeState = CrossFadeState.showFirst;

  bool get isFirst => _crossFadeState == CrossFadeState.showFirst;

  AnimationController _controller;
  Animation _rotate;
  double _rad = 0;
  bool _rotated = false;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(
        duration: Duration(milliseconds: widget.duration), vsync: this)
      ..addListener(_listenAnim)
      ..addStatusListener(_listenStatus);
    if (widget.initiallyExpanded) {
      _crossFadeState = CrossFadeState.showSecond;
    }
    _rotate = CurvedAnimation(parent: _controller, curve: widget.foldCurve);
  }

  _listenAnim() => setState(
          () => _rad = (_rotated ? (1 - _rotate.value) : _rotate.value) * pi / 2);

  _listenStatus(AnimationStatus status) {
    if (status == AnimationStatus.completed) {
      _rotated = !_rotated;
    }
  }

  @override
  Widget build(BuildContext context) {
    var tile = widget.tile ?? Icon(Icons.keyboard_arrow_right);

    return Column(
      children: <Widget>[
        Stack(
          alignment: Alignment.center,
          children: <Widget>[
            GestureDetector(onTap: _togglePanel, child: widget.child),
            Positioned(
              right: widget.tileOffsetRight,
              child: Transform(
                transform: Matrix4.rotationZ(_rad),
                alignment: Alignment.center,
                child: tile,
              ),
            )
          ],
        ),
        _buildPanel()
      ],
    );
  }

  void _togglePanel() {
    _controller.reset();
    _controller.forward();
    if (widget.onExpansionChanged != null) widget.onExpansionChanged(isFirst);
    setState(() {
      _crossFadeState =
      !isFirst ? CrossFadeState.showFirst : CrossFadeState.showSecond;
    });
  }

  Widget _buildPanel() => AnimatedCrossFade(
    sizeCurve: widget.foldCurve,
    firstChild: Container(),
    secondChild:
    Column(mainAxisSize: MainAxisSize.min, children: widget.children),
    duration: Duration(milliseconds: widget.duration),
    crossFadeState: _crossFadeState,
  );
}
